@@ -77,6 +77,8 @@ private:
     static QScriptValue setCurrentActivity(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue currentActivity(QScriptContext *controller, QScriptEngine *engine);
     static QScriptValue activities(QScriptContext *context, QScriptEngine *engine);
+    static QScriptValue setActivityName(QScriptContext *context, QScriptEngine *engine);
+    static QScriptValue activityName(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue newPanel(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue desktopsForActivity(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue desktops(QScriptContext *context, QScriptEngine *engine);
